@@ -20,10 +20,12 @@
 #include <sys/utime.h> /* for utime() */
 #include <io.h> /* for chmod() */
 #else
+#include <sys/types.h> /* some flavors of BSD (like OS X) require this to get time_t */
 #include <utime.h> /* for utime() */
 #include <unistd.h> /* for chown() */
 #endif
 #include <sys/stat.h> /* for stat() */
+#include <string.h> /* for strrchr() */
 #include "file.h"
 
 void flac__file_copy_metadata(const char *srcpath, const char *destpath)
@@ -42,3 +44,26 @@ void flac__file_copy_metadata(const char *srcpath, const char *destpath)
 #endif
 	}
 }
+
+off_t flac__file_get_filesize(const char *srcpath)
+{
+	struct stat srcstat;
+
+	if(0 == stat(srcpath, &srcstat))
+		return srcstat.st_size;
+	else
+		return -1;
+}
+
+const char *flac__file_get_basename(const char *srcpath)
+{
+	const char *p;
+
+	p = strrchr(srcpath, '\\');
+	if(0 == p) {
+		p = strrchr(srcpath, '/');
+		if(0 == p)
+			return srcpath;
+	}
+	return ++p;
+}
