@@ -24,6 +24,8 @@
 #ifndef _MSC_VER
 /* unlink is in stdio.h in VC++ */
 #include <unistd.h> /* for unlink() */
+#else
+#define strcasecmp stricmp
 #endif
 #include "FLAC/all.h"
 #include "analyze.h"
@@ -32,28 +34,31 @@
 #include "file.h"
 
 static int usage(const char *message, ...);
-static int encode_file(const char *infilename, const char *forced_outfilename);
+static int encode_file(const char *infilename, const char *forced_outfilename, FLAC__bool is_last_file);
 static int decode_file(const char *infilename, const char *forced_outfilename);
 
-bool verify = false, verbose = true, lax = false, test_only = false, analyze = false;
-bool do_mid_side = true, loose_mid_side = false, do_exhaustive_model_search = false, do_qlp_coeff_prec_search = false;
-bool force_to_stdout = false, delete_input = false;
-const char *cmdline_forced_outfilename = 0;
+FLAC__bool verify = false, verbose = true, lax = false, test_only = false, analyze = false;
+FLAC__bool do_mid_side = true, loose_mid_side = false, do_exhaustive_model_search = false, do_escape_coding = false, do_qlp_coeff_prec_search = false;
+FLAC__bool force_to_stdout = false, delete_input = false, sector_align = false;
+const char *cmdline_forced_outfilename = 0, *output_prefix = 0;
 analysis_options aopts = { false, false };
 unsigned padding = 0;
 unsigned max_lpc_order = 8;
 unsigned qlp_coeff_precision = 0;
-uint64 skip = 0;
+FLAC__uint64 skip = 0;
 int format_is_wave = -1, format_is_big_endian = -1, format_is_unsigned_samples = false;
 int format_channels = -1, format_bps = -1, format_sample_rate = -1;
 int blocksize = -1, min_residual_partition_order = -1, max_residual_partition_order = -1, rice_parameter_search_dist = -1;
 char requested_seek_points[50000]; /* @@@ bad MAGIC NUMBER */
 int num_requested_seek_points = -1; /* -1 => no -S options were given, 0 => -S- was given */
+FLAC__int32 align_reservoir_0[588], align_reservoir_1[588]; /* for carrying over samples from --sector-align */
+FLAC__int32 *align_reservoir[2] = { align_reservoir_0, align_reservoir_1 };
+unsigned align_reservoir_samples = 0; /* 0 .. 587 */
 
 int main(int argc, char *argv[])
 {
 	int i, retval = 0;
-	bool mode_decode = false;
+	FLAC__bool mode_decode = false;
 
 	if(argc <= 1)
 		return usage(0);
@@ -93,8 +98,14 @@ int main(int argc, char *argv[])
 			delete_input = true;
 		else if(0 == strcmp(argv[i], "--delete-input-file-"))
 			delete_input = false;
+		else if(0 == strcmp(argv[i], "--output-prefix"))
+			output_prefix = argv[++i];
+		else if(0 == strcmp(argv[i], "--sector-align"))
+			sector_align = true;
+		else if(0 == strcmp(argv[i], "--sector-align-"))
+			sector_align = false;
 		else if(0 == strcmp(argv[i], "--skip"))
-			skip = (uint64)atoi(argv[++i]); /* @@@ takes a pretty damn big file to overflow atoi() here, but it could happen */
+			skip = (FLAC__uint64)atoi(argv[++i]); /* @@@ takes a pretty damn big file to overflow atoi() here, but it could happen */
 		else if(0 == strcmp(argv[i], "--lax"))
 			lax = true;
 		else if(0 == strcmp(argv[i], "--lax-"))
@@ -105,6 +116,10 @@ int main(int argc, char *argv[])
 			do_exhaustive_model_search = true;
 		else if(0 == strcmp(argv[i], "-e-"))
 			do_exhaustive_model_search = false;
+		else if(0 == strcmp(argv[i], "-E"))
+			do_escape_coding = true;
+		else if(0 == strcmp(argv[i], "-E-"))
+			do_escape_coding = false;
 		else if(0 == strcmp(argv[i], "-l"))
 			max_lpc_order = atoi(argv[++i]);
 		else if(0 == strcmp(argv[i], "-m")) {
@@ -170,6 +185,7 @@ int main(int argc, char *argv[])
 			aopts.do_residual_text = false;
 		else if(0 == strcmp(argv[i], "-0")) {
 			do_exhaustive_model_search = false;
+			do_escape_coding = false;
 			do_mid_side = false;
 			loose_mid_side = false;
 			qlp_coeff_precision = 0;
@@ -179,6 +195,7 @@ int main(int argc, char *argv[])
 		}
 		else if(0 == strcmp(argv[i], "-1")) {
 			do_exhaustive_model_search = false;
+			do_escape_coding = false;
 			do_mid_side = true;
 			loose_mid_side = true;
 			qlp_coeff_precision = 0;
@@ -188,6 +205,7 @@ int main(int argc, char *argv[])
 		}
 		else if(0 == strcmp(argv[i], "-2")) {
 			do_exhaustive_model_search = false;
+			do_escape_coding = false;
 			do_mid_side = true;
 			loose_mid_side = false;
 			qlp_coeff_precision = 0;
@@ -198,6 +216,7 @@ int main(int argc, char *argv[])
 		}
 		else if(0 == strcmp(argv[i], "-3")) {
 			do_exhaustive_model_search = false;
+			do_escape_coding = false;
 			do_mid_side = false;
 			loose_mid_side = false;
 			qlp_coeff_precision = 0;
@@ -207,6 +226,7 @@ int main(int argc, char *argv[])
 		}
 		else if(0 == strcmp(argv[i], "-4")) {
 			do_exhaustive_model_search = false;
+			do_escape_coding = false;
 			do_mid_side = true;
 			loose_mid_side = true;
 			qlp_coeff_precision = 0;
@@ -216,6 +236,7 @@ int main(int argc, char *argv[])
 		}
 		else if(0 == strcmp(argv[i], "-5")) {
 			do_exhaustive_model_search = false;
+			do_escape_coding = false;
 			do_mid_side = true;
 			loose_mid_side = false;
 			qlp_coeff_precision = 0;
@@ -225,6 +246,7 @@ int main(int argc, char *argv[])
 		}
 		else if(0 == strcmp(argv[i], "-6")) {
 			do_exhaustive_model_search = false;
+			do_escape_coding = false;
 			do_mid_side = true;
 			loose_mid_side = false;
 			qlp_coeff_precision = 0;
@@ -235,6 +257,7 @@ int main(int argc, char *argv[])
 		}
 		else if(0 == strcmp(argv[i], "-7")) {
 			do_exhaustive_model_search = true;
+			do_escape_coding = false;
 			do_mid_side = true;
 			loose_mid_side = false;
 			qlp_coeff_precision = 0;
@@ -245,6 +268,7 @@ int main(int argc, char *argv[])
 		}
 		else if(0 == strcmp(argv[i], "-8")) {
 			do_exhaustive_model_search = true;
+			do_escape_coding = false;
 			do_mid_side = true;
 			loose_mid_side = false;
 			qlp_coeff_precision = 0;
@@ -255,6 +279,7 @@ int main(int argc, char *argv[])
 		}
 		else if(0 == strcmp(argv[i], "-9")) {
 			do_exhaustive_model_search = true;
+			do_escape_coding = true;
 			do_mid_side = true;
 			loose_mid_side = false;
 			do_qlp_coeff_prec_search = true;
@@ -322,6 +347,23 @@ int main(int argc, char *argv[])
 		return usage("ERROR: invalid value for -q '%u', must be 0 or >= %u\n", qlp_coeff_precision, FLAC__MIN_QLP_COEFF_PRECISION);
 	}
 
+	if(sector_align) {
+		if(mode_decode)
+			return usage("ERROR: --sector-align only allowed for encoding\n");
+		else if(skip > 0)
+			return usage("ERROR: --sector-align not allowed with --skip\n");
+		else if(format_channels >= 0 && format_channels != 2)
+			return usage("ERROR: --sector-align can only be done with stereo input\n");
+		else if(format_sample_rate >= 0 && format_sample_rate != 2)
+			return usage("ERROR: --sector-align can only be done with sample rate of 44100\n");
+	}
+	if(argc - i > 1 && cmdline_forced_outfilename) {
+		return usage("ERROR: -o cannot be used with multiple files\n");
+	}
+	if(cmdline_forced_outfilename && output_prefix) {
+		return usage("ERROR: --output-prefix conflicts with -o\n");
+	}
+
 	if(verbose) {
 		fprintf(stderr, "\n");
 		fprintf(stderr, "flac %s, Copyright (C) 2000,2001 Josh Coalson\n", FLAC__VERSION_STRING);
@@ -329,10 +371,10 @@ int main(int argc, char *argv[])
 		fprintf(stderr, "welcome to redistribute it under certain conditions.  Type `flac' for details.\n\n");
 
 		if(!mode_decode) {
-			fprintf(stderr, "options:%s%s -P %u -b %u%s -l %u%s%s -q %u -r %u,%u -R %u%s\n",
-				delete_input?" --delete-input-file":"", lax?" --lax":"",
+			fprintf(stderr, "options:%s%s%s -P %u -b %u%s -l %u%s%s%s -q %u -r %u,%u -R %u%s\n",
+				delete_input?" --delete-input-file":"", sector_align?" --sector-align":"", lax?" --lax":"",
 				padding, (unsigned)blocksize, loose_mid_side?" -M":do_mid_side?" -m":"", max_lpc_order,
-				do_exhaustive_model_search?" -e":"", do_qlp_coeff_prec_search?" -p":"",
+				do_exhaustive_model_search?" -e":"", do_escape_coding?" -E":"", do_qlp_coeff_prec_search?" -p":"",
 				qlp_coeff_precision,
 				(unsigned)min_residual_partition_order, (unsigned)max_residual_partition_order, (unsigned)rice_parameter_search_dist,
 				verify? " -V":""
@@ -341,7 +383,7 @@ int main(int argc, char *argv[])
 	}
 
 	if(mode_decode) {
-		bool first = true;
+		FLAC__bool first = true;
 		int save_format;
 
 		if(i == argc) {
@@ -361,11 +403,11 @@ int main(int argc, char *argv[])
 		}
 	}
 	else { /* encode */
-		bool first = true;
+		FLAC__bool first = true;
 		int save_format;
 
 		if(i == argc) {
-			retval = encode_file("-", 0);
+			retval = encode_file("-", 0, true);
 		}
 		else {
 			if(i + 1 != argc)
@@ -374,7 +416,7 @@ int main(int argc, char *argv[])
 				if(0 == strcmp(argv[i], "-") && !first)
 					continue;
 				save_format = format_is_wave;
-				retval = encode_file(argv[i], 0);
+				retval = encode_file(argv[i], 0, i == (argc-1));
 				format_is_wave = save_format;
 				first = false;
 			}
@@ -451,6 +493,7 @@ int usage(const char *message, ...)
 	fprintf(stderr, "  --a-rgp : generate gnuplot files of residual distribution of each subframe\n");
 	fprintf(stderr, "encoding options:\n");
 	fprintf(stderr, "  --lax : allow encoder to generate non-Subset files\n");
+	fprintf(stderr, "  --sector-align : align encoding of multiple files on sector boundaries\n");
 	fprintf(stderr, "  -S { # | X | #x } : include a point or points in a SEEKTABLE\n");
 	fprintf(stderr, "       #  : a specific sample number for a seek point\n");
 	fprintf(stderr, "       X  : a placeholder point (always goes at the end of the SEEKTABLE)\n");
@@ -482,8 +525,9 @@ int usage(const char *message, ...)
 	fprintf(stderr, "  -6   : synonymous with -l 8 -b 4608 -m -r 4\n");
 	fprintf(stderr, "  -7   : synonymous with -l 8 -b 4608 -m -e -r 6\n");
 	fprintf(stderr, "  -8   : synonymous with -l 12 -b 4608 -m -e -r 6\n");
-	fprintf(stderr, "  -9   : synonymous with -l 32 -b 4608 -m -e -r 16 -p (very slow!)\n");
+	fprintf(stderr, "  -9   : synonymous with -l 32 -b 4608 -m -e -E -r 16 -p (very slow!)\n");
 	fprintf(stderr, "  -e   : do exhaustive model search (expensive!)\n");
+	fprintf(stderr, "  -E   : include escape coding in the entropy coder\n");
 	fprintf(stderr, "  -l # : specify max LPC order; 0 => use only fixed predictors\n");
 	fprintf(stderr, "  -p   : do exhaustive search of LP coefficient quantization (expensive!);\n");
 	fprintf(stderr, "         overrides -q, does nothing if using -l 0\n");
@@ -494,8 +538,8 @@ int usage(const char *message, ...)
 	fprintf(stderr, "  -R # : Rice parameter search distance (# is 0..32; above 2 doesn't help much)\n");
 	fprintf(stderr, "  -V   : verify a correct encoding by decoding the output in parallel and\n");
 	fprintf(stderr, "         comparing to the original\n");
-	fprintf(stderr, "  -S-, -m-, -M-, -e-, -p-, -V-, --delete-input-file-, --lax- can all be used to\n");
-	fprintf(stderr, "  turn off a particular option\n");
+	fprintf(stderr, "  -S-, -m-, -M-, -e-, -E-, -p-, -V-, --delete-input-file-, --lax-, --sector-align-\n");
+	fprintf(stderr, "  can all be used to turn off a particular option\n");
 	fprintf(stderr, "format options:\n");
 	fprintf(stderr, "  -fb | -fl : big-endian | little-endian byte order\n");
 	fprintf(stderr, "  -fc channels\n");
@@ -504,45 +548,52 @@ int usage(const char *message, ...)
 	fprintf(stderr, "  -fu : unsigned samples (default is signed)\n");
 	fprintf(stderr, "  -fr : force to raw format (even if filename ends in .wav)\n");
 	fprintf(stderr, "  -fw : force to RIFF WAVE\n");
-	return 1;
+
+	return message? 1 : 0;
 }
 
-int encode_file(const char *infilename, const char *forced_outfilename)
+int encode_file(const char *infilename, const char *forced_outfilename, FLAC__bool is_last_file)
 {
 	FILE *encode_infile;
 	char outfilename[4096]; /* @@@ bad MAGIC NUMBER */
 	char *p;
+	FLAC__byte lookahead[12];
+	unsigned lookahead_length = 0;
 	int retval;
+	long infilesize;
 
 	if(0 == strcmp(infilename, "-")) {
+		infilesize = -1;
 		encode_infile = stdin;
 	}
 	else {
+		infilesize = flac__file_get_filesize(infilename);
 		if(0 == (encode_infile = fopen(infilename, "rb"))) {
 			fprintf(stderr, "ERROR: can't open input file %s\n", infilename);
 			return 1;
 		}
 	}
 
-	if(verbose)
-		fprintf(stderr, "%s:\n", infilename);
+	if(sector_align && !format_is_wave && infilesize < 0) {
+		fprintf(stderr, "ERROR: can't --sector-align when the input size is unknown\n");
+		return 1;
+	}
 
 	if(format_is_wave < 0) {
-		/* lamely attempt to guess the file type based on the first 4 bytes (which is all ungetc will guarantee us) */
-		char head[4];
-		int h, n;
 		/* first set format based on name */
-		if(strstr(infilename, ".wav") == infilename + (strlen(infilename) - strlen(".wav")))
+		if(0 == strcasecmp(infilename+(strlen(infilename)-4), ".wav"))
 			format_is_wave = true;
 		else
 			format_is_wave = false;
-		if((n = fread(head, 1, 4, encode_infile)) < 4) {
+
+		/* attempt to guess the file type based on the first 12 bytes */
+		if((lookahead_length = fread(lookahead, 1, 12, encode_infile)) < 12) {
 			if(format_is_wave)
 				fprintf(stderr, "WARNING: %s is not a WAVE file, treating as a raw file\n", infilename);
 			format_is_wave = false;
 		}
 		else {
-			if(strncmp(head, "RIFF", 4)) {
+			if(strncmp(lookahead, "RIFF", 4) || strncmp(lookahead+8, "WAVE", 4)) {
 				if(format_is_wave)
 					fprintf(stderr, "WARNING: %s is not a WAVE file, treating as a raw file\n", infilename);
 				format_is_wave = false;
@@ -550,8 +601,6 @@ int encode_file(const char *infilename, const char *forced_outfilename)
 			else
 				format_is_wave = true;
 		}
-		for(h = n-1; h >= 0; h--)
-			ungetc(head[h], encode_infile);
 	}
 
 	if(!format_is_wave) {
@@ -562,7 +611,8 @@ int encode_file(const char *infilename, const char *forced_outfilename)
 	if(encode_infile == stdin || force_to_stdout)
 		strcpy(outfilename, "-");
 	else {
-		strcpy(outfilename, infilename);
+		strcpy(outfilename, output_prefix? output_prefix : "");
+		strcat(outfilename, infilename);
 		if(0 == (p = strrchr(outfilename, '.')))
 			strcat(outfilename, ".flac");
 		else {
@@ -578,9 +628,9 @@ int encode_file(const char *infilename, const char *forced_outfilename)
 		forced_outfilename = cmdline_forced_outfilename;
 
 	if(format_is_wave)
-		retval = flac__encode_wav(encode_infile, infilename, forced_outfilename, verbose, skip, verify, lax, do_mid_side, loose_mid_side, do_exhaustive_model_search, do_qlp_coeff_prec_search, min_residual_partition_order, max_residual_partition_order, rice_parameter_search_dist, max_lpc_order, (unsigned)blocksize, qlp_coeff_precision, padding, requested_seek_points, num_requested_seek_points);
+		retval = flac__encode_wav(encode_infile, infilesize, infilename, forced_outfilename, lookahead, lookahead_length, align_reservoir, &align_reservoir_samples, sector_align, is_last_file, verbose, skip, verify, lax, do_mid_side, loose_mid_side, do_exhaustive_model_search, do_escape_coding, do_qlp_coeff_prec_search, min_residual_partition_order, max_residual_partition_order, rice_parameter_search_dist, max_lpc_order, (unsigned)blocksize, qlp_coeff_precision, padding, requested_seek_points, num_requested_seek_points);
 	else
-		retval = flac__encode_raw(encode_infile, infilename, forced_outfilename, verbose, skip, verify, lax, do_mid_side, loose_mid_side, do_exhaustive_model_search, do_qlp_coeff_prec_search, min_residual_partition_order, max_residual_partition_order, rice_parameter_search_dist, max_lpc_order, (unsigned)blocksize, qlp_coeff_precision, padding, requested_seek_points, num_requested_seek_points, format_is_big_endian, format_is_unsigned_samples, format_channels, format_bps, format_sample_rate);
+		retval = flac__encode_raw(encode_infile, infilesize, infilename, forced_outfilename, lookahead, lookahead_length, is_last_file, verbose, skip, verify, lax, do_mid_side, loose_mid_side, do_exhaustive_model_search, do_escape_coding, do_qlp_coeff_prec_search, min_residual_partition_order, max_residual_partition_order, rice_parameter_search_dist, max_lpc_order, (unsigned)blocksize, qlp_coeff_precision, padding, requested_seek_points, num_requested_seek_points, format_is_big_endian, format_is_unsigned_samples, format_channels, format_bps, format_sample_rate);
 
 	if(retval == 0 && strcmp(infilename, "-")) {
 		if(strcmp(forced_outfilename, "-"))
@@ -589,7 +639,7 @@ int encode_file(const char *infilename, const char *forced_outfilename)
 			unlink(infilename);
 	}
 
-	return retval;
+	return 0;
 }
 
 int decode_file(const char *infilename, const char *forced_outfilename)
@@ -613,7 +663,8 @@ int decode_file(const char *infilename, const char *forced_outfilename)
 		strcpy(outfilename, "-");
 	else {
 		const char *suffix = suffixes[analyze? 2 : format_is_wave? 0 : 1];
-		strcpy(outfilename, infilename);
+		strcpy(outfilename, output_prefix? output_prefix : "");
+		strcat(outfilename, infilename);
 		if(0 == (p = strrchr(outfilename, '.')))
 			strcat(outfilename, suffix);
 		else {
@@ -642,5 +693,5 @@ int decode_file(const char *infilename, const char *forced_outfilename)
 			unlink(infilename);
 	}
 
-	return retval;
+	return 0;
 }
